from . import BaseApi, Region


class Clan(BaseApi):
    def __init__(self, token, api_link, clan_id: str = "", region: Region = Region.RU):
        super().__init__(token, api_link)

        if not clan_id:
            raise ValueError(f"Invalid clan_id: {clan_id}")

        else:
            self.clan_id = clan_id

        self.region = region

    def info(self):
        """
        Возвращает информацию о клане
        """

        method = f"{self.region.value}/clan/{self.clan_id}/info"
        return self._request(method)

    def members(self):
        """
        Возвращает участников клана
        """

        method = f"{self.region.value}/clan/{self.clan_id}/members"
        return self._request(method)

    def __repr__(self):
        return f"<Clan> clan_id='{self.clan_id}' region='{self.region}'"



from . import ApiLink

from typing import Any
import requests


class BaseApi:
    def __init__(self, token: str, api_link: str | ApiLink = "", warn_demo: bool=True):
        assert token is not None, "token should not be None"

        if not api_link:
            api_link = ApiLink.DEMO.value

        if warn_demo and api_link == ApiLink.DEMO.value:
            print("Warning, you are using a demo version of api")

        self.api_link = api_link
        self.token = token

    @property
    def token_part(self) -> str:
        return f"{self.token[0:15]}..."

    @property
    def headers(self) -> dict:
        """
        Возвращает header для авторизации
        """

        return {"Authorization": f"Bearer {self.token}"}

    def _request(self, method: str) -> Any:
        """
        Делает запрос в API
        """

        response = requests.get(
            f"{self.api_link}/{method}",
            headers=self.headers
        )
        return response.json()

    def _offset_and_limit(self, offset: int, limit: int) -> None:
        """
        Проверяет правильность значений offset и limit
        """

        assert offset >= 0, f"offset should be >= 0, got {offset}"
        assert limit in range(0, 101), f"limit should be between 0 and 100, got {limit}"

    def __repr__(self):
        return f"<BaseAPI> api_link='{self.api_link}' token='{self.token_part}'"



from . import BaseApi, Region, Sort, Order


class Auction(BaseApi):
    def __init__(self, token, api_link, item_id: str = "", region: Region = Region.RU):
        super().__init__(token, api_link)

        if not item_id:
            raise ValueError(f"Invalid item '{item_id}'")

        else:
            self.item_id = item_id

        self.region = region

    def history(self, offset: int=0, limit: int=20):
        """
        offset: Количество цен в списке, которое нужно пропустить, по умолчанию 0 \n
        limit: Количество возвращаемых цен, начиная со смещения, минимум 0, максимум 100, по умолчанию 20 \n
        """

        self._offset_and_limit(offset, limit)

        method = f"{self.region.value}/auction/{self.item_id}/history?offset={offset}&limit={limit}"
        return self._request(method)

    def lots(self, offset: int=0, limit: int=20, sort: Sort = Sort.TIME_CREATED, order: Order = Order.ASCENDING):
        """
        offset: Количество цен в списке, которое нужно пропустить, по умолчанию 0 \n
        limit: Количество возвращаемых цен, начиная со смещения, минимум 0, максимум 100, по умолчанию 20 \n
        sort: Параметр для сортировки по одному из: TIME_CREATED, TIME_LEFT, CURRENT_PRICE, BUYOUT_PRICE \n
        order: Параметр для порядка: ASCENDING (возрастания), DESCENDING (убывания) \n
        """

        self._offset_and_limit(offset, limit)

        method = f"{self.region.value}/auction/{self.item_id}/lots?offset={offset}&limit={limit}&sort={sort.value}&order={order.value}"
        return self._request(method)

    def __repr__(self):
        return f"<Clan> item='{self.item_id}' region='{self.region}'"



from enum import Enum


class ApiLink(Enum):
    """
    DEMO: Демонстрационный API
    PRODUCTION: Полноценный API
    """

    DEMO = "http://dapi.stalcraft.net"
    PRODUCTION = "http://eapi.stalcraft.net"


class Region(Enum):
    """
    RU: Russia / Россия
    EU: Europe / Европа
    NA: North America / Северная Америка
    SEA: South East Asia / Юго-Восточная Азия
    """
    RU = "ru"
    EU = "eu"
    NA = "na"
    SEA = "sea"


class Sort(Enum):
    """
    TIME_CREATED: Время создания
    TIME_LEFT: Времени осталось
    CURRENT_PRICE: Цена ставки
    BUYOUT_PRICE: Цена выкупа
    """

    TIME_CREATED = "time_created"
    TIME_LEFT = "time_left"
    CURRENT_PRICE = "current_price"
    BUYOUT_PRICE = "buyout_price"


class Order(Enum):
    """
    ASCENDING: Возростание
    DESCENDING: Убывание
    """

    ASCENDING = "asc"
    DESCENDING = "desc"



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.3'
DESCRIPTION = 'stalcraft-api unofficial python library'

# Setting up
setup(
    name="stalcraft-api",
    version=VERSION,
    author="onejeuu",
    author_email="<bloodtrail@beber1k.ru>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    keywords=['python', 'stalcraft', 'api'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows",
    ]
)



from . import BaseApi, ApiLink, Auction, Clan, Region


class Client(BaseApi):
    def __init__(self, token: str, api_link: str | ApiLink = "", warn_demo: bool=True):
        """
        token: Токен для авторизации
        api_link: Опциональный параметр, ссылка на API
        warn_demo: Предупреждать о том что используется demo версия API
        """

        super().__init__(token, api_link, warn_demo)

    def regions(self):
        """
        Возвращает список регионов
        """

        method = "regions"
        return self._request(method)

    def characters(self, region: Region = Region.RU):
        """
        Возвращает список персонажей

        region: Регион сталкрафта, по умолчанию Region.RU
        """

        method = f"{region.value}/characters"
        return self._request(method)

    def clans(self, offset: int=0, limit: int=20, region: Region = Region.RU):
        """
        Возвращает список кланов

        offset: Количество цен в списке, которое нужно пропустить, по умолчанию 0 \n
        limit: Количество возвращаемых цен, начиная со смещения, минимум 0, максимум 100, по умолчанию 20 \n
        region: Регион сталкрафта, по умолчанию Region.RU
        """

        self._offset_and_limit(offset, limit)

        method = f"{region.value}/clans?offset={offset}&limit={limit}"
        return self._request(method)

    def emission(self, region: Region = Region.RU):
        """
        Возвращает информацию о выбросе

        region: Регион сталкрафта, по умолчанию Region.RU
        """

        method = f"{region.value}/emission"
        return self._request(method)

    def friends(self, character: str = "", region: Region = Region.RU):
        """
        Возвращает список друзей

        character: Имя персонажа
        region: Регион сталкрафта, по умолчанию Region.RU
        """

        if not character:
            raise ValueError(f"Invalid character '{character}'")

        method = f"{region.value}/friends/{character}"
        return self._request(method)

    def auction(self, item_id: str = "", region: Region = Region.RU):
        """
        Интерфейс для работы с аукционом

        item: ID предмета, к примеру "y1q9"
        region: Регион сталкрафта, по умолчанию Region.RU
        """

        return Auction(self.token, self.api_link, item_id, region)

    def clan(self, clan_id: str = "", region: Region = Region.RU):
        """
        Интерфейс для работы с кланами

        clan_id: ID клана, к примеру "647d6c53-b3d7-4d30-8d08-de874eb1d845"
        region: Регион сталкрафта, по умолчанию Region.RU
        """

        return Clan(self.token, self.api_link, clan_id, region)

    def __repr__(self):
        return f"<Client> api_link='{self.api_link}' token='{self.token_part}'"
